@@ -1,14 +1,14 @@
 #ifndef lint
-static char rcsId[]="$Header: /debian/home/gnomecvs/gnome-libs/gtk-xmhtml/fonts.c,v 1.6 1998/02/12 03:08:37 unammx Exp $";
+static char rcsId[]="$Header: /cvs/gnome/gnome-libs/gtk-xmhtml/fonts.c,v 1.7 1998/05/11 07:54:07 kmaraas Exp $";
 #endif
 /*****
 * fonts.c : XmHTML font loading & caching routines.
 *
-* This file Version	$Revision: 1.6 $
+* This file Version	$Revision: 1.7 $
 *
 * Creation date:		Mon Sep 22 09:46:00 GMT+0100 1997
-* Last modification: 	$Date: 1998/02/12 03:08:37 $
-* By:					$Author: unammx $
+* Last modification: 	$Date: 1998/05/11 07:54:07 $
+* By:					$Author: kmaraas $
 * Current State:		$State: Exp $
 *
 * Author:				newt
@@ -36,6 +36,12 @@ static char rcsId[]="$Header: /debian/home/gnomecvs/gnome-libs/gtk-xmhtml/fonts.
 /*****
 * ChangeLog 
 * $Log: fonts.c,v $
+* Revision 1.7  1998/05/11 07:54:07  kmaraas
+* 1998-05-11  Kjartan Maraas  <kmaraas@fib.hl.no>
+*
+* 	* Removed two compiler warnings in gnome-libs/gtk-xmhtml/
+* 	fonts.c and gtk-forms.c.
+*
 * Revision 1.6  1998/02/12 03:08:37  unammx
 * Merge to Koen's XmHTML 1.1.2 + following fixes:
 *
@@ -1524,7 +1530,7 @@ XmHTMLGetFontCacheInfo(TWidget w)
 	info->hits      = cache->hits;			/* no of hits */
 	info->misses    = cache->misses;		/* no of font cache misses */
 	info->nwidgets  = cache->nwidgets;		/* no of widgets using this cache */
-	info->widgets   = cache->widgets;
+	info->widgets   = (WidgetList)cache->widgets;
 	info->fonts     = (String*)calloc(info->nentries, sizeof(String));
 	info->mapping   = (String*)calloc(info->nentries, sizeof(String));
 
