@@ -20,14 +20,14 @@
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-/* #include <config.h> */
+#include <config.h>
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>	/* atoi() */
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <glib.h>
-#include "libgnome.h"
+#include "libgnomeP.h"
 
 #ifndef HAVE_STRNDUP
 /* Like strdup, but only copy N chars.  */
@@ -96,6 +96,23 @@ static TProfile *Current = 0;
  */
 static TProfile *Base = 0;
 
+/* a set handler is a function which is called every time a gnome_config_set_*
+   function is called, this can be used by the app to say guarantee a sync,
+   apps using libgnomeui should not call this, libgnomeui already provides
+   this, this would be for non-gui apps and apps that use a different toolkit
+   then gtk*/
+static void (*set_handler)(void *data) = NULL;
+static void *set_handler_data;
+
+#define CALL_SET_HANDLER() { if(set_handler) (*set_handler)(set_handler_data); }
+
+/* same as above for a "sync" handler */
+static void (*sync_handler)(void *data) = NULL;
+static void *sync_handler_data;
+
+#define CALL_SYNC_HANDLER() { if(sync_handler) (*sync_handler)(sync_handler_data); }
+
+
 static void
 release_path (ParsedPath *p)
 {
@@ -571,6 +588,7 @@ void
 gnome_config_sync (void)
 {
 	dump_profile (Base);
+	CALL_SYNC_HANDLER();
 }
 
 static void 
@@ -869,74 +887,89 @@ _gnome_config_get_int_with_default (const char *path, gboolean *def, gint priv)
 	r = access_config (LOOKUP, pp->section, pp->key, pp->def, pp->file,
 			   def);
 
-	g_return_val_if_fail(r != NULL, 0);
+	/* It isn't an error if the key is not found.  */
+	if (r == NULL)
+		return 0;
 
 	v = atoi (r);
 	release_path (pp);
 	return v;
 }
 
+gdouble
+_gnome_config_get_float_with_default (const char *path, gboolean *def, gint priv)
+{
+	ParsedPath *pp;
+	const char *r;
+	gdouble v;
+	
+	pp = parse_path (path, priv);
+	r = access_config (LOOKUP, pp->section, pp->key, pp->def, pp->file,
+			   def);
+
+	/* It isn't an error if the key is not found.  */
+	if (r == NULL)
+		return 0;
+
+	v = strtod(r, NULL);
+	release_path (pp);
+	return v;
+}
+
 char *
 _gnome_config_get_translated_string_with_default (const char *path,
 						  gboolean *def,
 						  gint priv)
 {
-  GList *language_list;
+	GList *language_list;
 
-  char *value= NULL;
+	char *value= NULL;
 
-  language_list= gnome_i18n_get_language_list ("LC_ALL");
-  
-  while (!value && language_list)
-    {
-      const char *lang= language_list->data;
+	language_list= gnome_i18n_get_language_list ("LC_ALL");
 
-      if (strcmp (lang, "C") == 0)
-	{
-	  value = _gnome_config_get_string_with_default (path, def, priv);
-	  if (value)
-	    return value;
-	  else
-	    {
-	      /* FIXME: this "else" block should go away in some time.
-	       * We use it to handle the old broken files that were
-	       * written by the old buggy set_translated_string
-	       * function, which *did* append the "[C]" suffix.
-	       */
+	while (!value && language_list) {
+		const char *lang= language_list->data;
 
-	      gchar *tkey;
+		if (strcmp (lang, "C") == 0) {
+			value = _gnome_config_get_string_with_default (path, def, priv);
+			if (value)
+				return value;
+		} else {
+			gchar *tkey;
 
-	      tkey = g_copy_strings (path, "[C]", NULL);
-	      value = _gnome_config_get_string_with_default (tkey, def, priv);
-	      g_free (tkey);
+			tkey= g_copy_strings (path, "[", lang, "]", NULL);
+			value= _gnome_config_get_string_with_default (tkey, def, priv);
+			g_free (tkey);
 
-	      if (!value || *value == '\0')
-		{
-		  g_free (value);
-		  value = NULL;
+			if (!value || *value == '\0') {
+				size_t n;
+
+				g_free (value);
+				value= NULL;
+
+				/* Sometimes the locale info looks
+				   like `pt_PT@verbose'.  In this case
+				   we want to try `pt' as a backup.  */
+				n = strcspn (lang, "@_");
+				if (lang[n]) {
+					char *copy = strndup (lang, n);
+					tkey = g_copy_strings (path, "[",
+							       copy, "]",
+							       NULL);
+					value = _gnome_config_get_string_with_default (tkey, def, priv);
+					g_free (tkey);
+					g_free (copy);
+					if (! value || *value == '\0') {
+						g_free (value);
+						value = NULL;
+					}
+				}
+			}
 		}
-
-	      return value;
-	    }
+		language_list= language_list->next;
 	}
-      else
-	{
-	  gchar *tkey;
-	  
-	  tkey= g_copy_strings (path, "[", lang, "]", NULL);
-	  value= _gnome_config_get_string_with_default (tkey, def, priv);
-	  g_free (tkey);
-	  
-	  if (!value || *value == '\0')
-	    {
-	      g_free (value);
-	      value= NULL;
-	    }
-	}
-      language_list= language_list->next;
-    }
 
-  return value;
+	return value;
 }
 
 char *
@@ -968,7 +1001,9 @@ _gnome_config_get_bool_with_default (const char *path, gboolean *def,
 	r = access_config (LOOKUP, pp->section, pp->key, pp->def, pp->file,
 			   def);
 
-	g_return_val_if_fail(r != NULL, 0);
+	/* It isn't an error if the key is not found.  */
+	if (r == NULL)
+		return 0;
 
 	if (!strcasecmp (r, "true")){
 		v = 1;
@@ -1055,22 +1090,21 @@ void
 _gnome_config_set_translated_string (const char *path, const char *value,
 				     gint priv)
 {
-  GList *language_list;
-  const char *lang;
-  char *tkey;
+	GList *language_list;
+	const char *lang;
+	char *tkey;
 
-  language_list= gnome_i18n_get_language_list("LC_ALL");
+	language_list= gnome_i18n_get_language_list("LC_ALL");
 
-  lang= language_list ? language_list->data : NULL;
-  
-  if (lang && (strcmp (lang, "C") != 0))
-    {
-      tkey = g_copy_strings (path, "[", lang, "]", NULL);
-      _gnome_config_set_string(tkey, value, priv);
-      g_free (tkey);
-    }
-  else
-    _gnome_config_set_string (path, value, priv);
+	lang= language_list ? language_list->data : NULL;
+
+	if (lang && (strcmp (lang, "C") != 0)) {
+		tkey = g_copy_strings (path, "[", lang, "]", NULL);
+		_gnome_config_set_string(tkey, value, priv);
+		g_free (tkey);
+	} else
+		_gnome_config_set_string (path, value, priv);
+	CALL_SET_HANDLER();
 }
 
 void
@@ -1083,6 +1117,7 @@ _gnome_config_set_string (const char *path, const char *new_value, gint priv)
 	r = access_config (SET, pp->section, pp->key, new_value, pp->file,
 			   NULL);
 	release_path (pp);
+	CALL_SET_HANDLER();
 }
 
 void
@@ -1097,6 +1132,22 @@ _gnome_config_set_int (const char *path, int new_value, gint priv)
 	r = access_config (SET, pp->section, pp->key, intbuf, pp->file,
 			   NULL);
 	release_path (pp);
+	CALL_SET_HANDLER();
+}
+
+void
+_gnome_config_set_float (const char *path, gdouble new_value, gint priv)
+{
+	ParsedPath *pp;
+	char floatbuf [40];
+	const char *r;
+	
+	pp = parse_path (path, priv);
+	sprintf (floatbuf, "%.17g", new_value);
+	r = access_config (SET, pp->section, pp->key, floatbuf, pp->file,
+			   NULL);
+	release_path (pp);
+	CALL_SET_HANDLER();
 }
 
 void
@@ -1109,6 +1160,7 @@ _gnome_config_set_bool (const char *path, gboolean new_value, gint priv)
 	r = access_config (SET, pp->section, pp->key,
 			   new_value ? "true" : "false", pp->file, NULL);
 	release_path (pp);
+	CALL_SET_HANDLER();
 }
 
 char *
@@ -1152,22 +1204,36 @@ _gnome_config_set_vector (const char *path, int argc,
 	access_config (SET, pp->section, pp->key, s, pp->file, NULL);
 	g_free (s);
 	release_path (pp);
+	CALL_SET_HANDLER();
 }
 
 void
 gnome_config_push_prefix (const char *path)
 {
-  prefix_list = g_slist_prepend(prefix_list, g_strdup(path));
+	prefix_list = g_slist_prepend(prefix_list, g_strdup(path));
 }
 
 void
 gnome_config_pop_prefix (void)
 {
-  if(prefix_list)
-    {
-      g_free(prefix_list->data);
-      prefix_list = g_slist_remove_link(prefix_list, prefix_list);
-    }
+	if(prefix_list) {
+		g_free(prefix_list->data);
+		prefix_list = g_slist_remove_link(prefix_list, prefix_list);
+	}
+}
+
+void
+gnome_config_set_set_handler(void (*func)(void *),void *data)
+{
+	set_handler = func;
+	set_handler_data = data;
+}
+
+void
+gnome_config_set_sync_handler(void (*func)(void *),void *data)
+{
+	sync_handler = func;
+	sync_handler_data = data;
 }
 
 #ifdef TEST
@@ -1198,128 +1264,3 @@ main ()
 	
 }
 #endif
-
-#ifndef AVOID_THIS_EXTRA_COOL_HACK
-
-/* THIS WILL BE GONE SOON (Can be removed at any time, nothing but old
- * binaries depends on this).
- */
-#undef gnome_config_get_string_with_default
-#undef gnome_config_get_translated_string_with_default
-#undef gnome_config_get_int_with_default
-#undef gnome_config_get_bool_with_default
-#undef gnome_config_get_vector_with_default
-
-char *
-gnome_config_get_string_with_default (const char *path, gboolean *def)
-{
-	return _gnome_config_get_string_with_default(path,def,FALSE);
-}
-
-char *
-gnome_config_get_translated_string_with_default (const char *path, gboolean *def)
-{
-	return _gnome_config_get_translated_string_with_default(path, def, FALSE);
-}
-
-gint
-gnome_config_get_int_with_default (const char *path, gboolean *def)
-{
-	return _gnome_config_get_int_with_default(path, def, FALSE);
-}
-
-gboolean
-gnome_config_get_bool_with_default (const char *path, gboolean *def)
-{
-	return _gnome_config_get_bool_with_default (path, def, FALSE);
-}
-
-void
-gnome_config_get_vector_with_default     (const char *path, gint *argcp, char ***argvp, gboolean *def)
-{
-        _gnome_config_get_vector_with_default (path, argcp, argvp, def, FALSE);
-}
-
-#undef gnome_config_set_string
-#undef gnome_config_set_translated_string
-#undef gnome_config_set_int
-#undef gnome_config_set_bool
-#undef gnome_config_set_vector
-
-void
-gnome_config_set_string (const char *path, const char *value)
-{
-	_gnome_config_set_string (path, value, FALSE);
-}
-
-void
-gnome_config_set_translated_string(const char *path, const char *val)
-{
-	
-	_gnome_config_set_translated_string (path, val, FALSE);
-}
-
-void
-gnome_config_set_int(const char *path, int value)
-{
-	_gnome_config_set_int (path, value, FALSE);
-}
-
-void
-gnome_config_set_bool(const char *path, gboolean value)
-{
-	_gnome_config_set_bool (path, value, FALSE);
-}
-
-void
-gnome_config_set_vector(const char *path, int argc, const char * const argv[])
-{
-	_gnome_config_set_vector (path, argc, argv, FALSE);
-}
-
-#undef gnome_config_clean_key
-#undef gnome_config_clean_section
-#undef gnome_config_clean_file
-#undef gnome_config_has_section
-#undef gnome_config_init_iterator
-#undef gnome_config_init_iterator_sections
-
-void
-gnome_config_clean_key(const char *path)
-{
-	_gnome_config_clean_key(path,FALSE);
-}
-
-void
-gnome_config_clean_section(const char *path)
-{
-	_gnome_config_clean_section(path,FALSE);
-}
-
-void
-gnome_config_clean_file(const char *path)
-{
-	_gnome_config_clean_file(path,FALSE);
-}
-
-gboolean
-gnome_config_has_section(const char *path)
-{
-	return _gnome_config_has_section(path,FALSE);
-}
-
-
-void *
-gnome_config_init_iterator(const char *path)
-{
-	return _gnome_config_init_iterator(path,FALSE);
-}
-
-
-void *
-gnome_config_init_iterator_sections(const char *path)
-{
-	return _gnome_config_init_iterator_sections(path,FALSE);
-}
-
-#endif
