@@ -15,9 +15,16 @@
    License along with the Gnome Library; see the file COPYING.LIB.  If not,
    write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.  */
-#include <malloc.h>
+
+#include <config.h>
+#include <stdlib.h>
+#include <glib.h>
+
 #include "gnome-defs.h"
 #include "gnome-parse.h"
+#include "gnome-i18nP.h"
+
+#define GROUP -2
 
 static error_t default_parse_func (int key, char *arg,
 				   struct argp_state *state);
@@ -32,6 +39,25 @@ static int args_count;
 /* Vector of all registered child argument structures.  */
 static struct argp_child *arguments;
 
+static struct argp_option our_options[] =
+{
+	{ N_("\nGeneric options"), -1, NULL, OPTION_DOC,   NULL, GROUP },
+	{ NULL, 0, NULL, 0, NULL, 0 }
+};
+
+/* A helper argument parser that exists solely to get some
+   documentation in the --help output.  */
+static struct argp help_parser =
+{
+  our_options,
+  NULL,
+  NULL,
+  NULL,
+  NULL,
+  NULL,
+  NULL
+};
+
 /* Default argument parser.  */
 static struct argp default_parser =
 {
@@ -81,8 +107,12 @@ gnome_parse_register_arguments (const struct argp *parser)
   arguments[args_count].argp = parser;
   arguments[args_count].flags = 0;
   arguments[args_count].header = "";
-  /* FIXME: pick a better value here.  */
-  arguments[args_count].group = -2;
+  /* FIXME: pick a better value here.  The present value is chosen so
+     that the --help output will look right; in particular the
+     "Generic options" text must appear before --help.  Gross!  It
+     should be possible for a parser to override the group and header
+     fields.  */
+  arguments[args_count].group = GROUP - 1;
 
   ++args_count;
   arguments[args_count].argp = NULL;
@@ -94,6 +124,11 @@ gnome_parse_arguments (struct argp *parser,
 		       unsigned int flags,
 		       int *arg_index)
 {
+  gnome_parse_register_arguments (&help_parser);
+  /* A hack to make --help output look right.  */
+  arguments[args_count - 1].header = NULL;
+  arguments[args_count - 1].group = 0;
+
   if (! parser)
     parser = &default_parser;
   parser->children = arguments;
