@@ -12,6 +12,9 @@
 #include "libgnome/gnome-util.h"
 #include "libgnome/gnome-config.h"
 #include "libgnome/gnome-help.h"
+/* Note that this file must include gnome-i18n, and not gnome-i18nP,
+   so that _() is the same as the one seen by the application.  This
+   is moderately bogus; we should just call gettext() directly here.  */
 #include "libgnome/gnome-i18n.h"
 #include "gnome-app.h"
 #include "gnome-app-helper.h"
@@ -19,49 +22,55 @@
 #include "gnome-pixmap.h"
 #include "gnome-stock.h"
 #include <string.h>
+#include <stdio.h>
 #include <ctype.h>
 #include <gtk/gtk.h>
 
-static void gnome_app_do_menu_creation        (GnomeApp *app,
+void gnome_app_do_menu_creation        (GnomeApp *app,
 					       GtkWidget *parent_widget,
 					       gint pos,
 					       GnomeUIInfo *menuinfo,
-					       GnomeUIBuilderData uidata);
-static void gnome_app_do_ui_signal_connect    (GnomeApp *app,
+					       GnomeUIBuilderData *uidata);
+void gnome_app_do_ui_signal_connect    (GnomeApp *app,
 					       GnomeUIInfo *info_item,
 					       gchar *signal_name,
-					       GnomeUIBuilderData uidata);
+					       GnomeUIBuilderData *uidata);
 static void gnome_app_do_ui_accelerator_setup (GnomeApp *app,
 					       gchar *signal_name,
-				               GnomeUIInfo *menuinfo_item);
+				         GnomeUIInfo *menuinfo_item);
 static void gnome_app_do_toolbar_creation     (GnomeApp *app,
 					       GtkWidget *parent_widget,
 					       GnomeUIInfo *tbinfo,
-					       GnomeUIBuilderData uidata);
-static void gnome_app_add_help_menu_entries   (GnomeApp *app,
+					       GnomeUIBuilderData *uidata);
+static gint gnome_app_add_help_menu_entries   (GnomeApp *app,
 					       GtkWidget *parent_widget,
 					       gint pos,
-				               GnomeUIInfo *menuinfo_item);
-static void gnome_app_add_radio_menu_entries  (GnomeApp *app,
+                 GnomeUIInfo *menuinfo_item);
+static gint gnome_app_add_radio_menu_entries  (GnomeApp *app,
 					       GtkWidget *parent_widget,
 					       gint pos,
 					       GnomeUIInfo *menuinfo,
-					       GnomeUIBuilderData uidata);
+					       GnomeUIBuilderData *uidata);
 static void gnome_app_add_radio_toolbar_entries(GnomeApp *app,
 						GtkWidget *parent_widget,
 						GnomeUIInfo *tbinfo,
-						GnomeUIBuilderData uidata);
+						GnomeUIBuilderData *uidata);
 
-static void
+void
 gnome_app_do_menu_creation(GnomeApp *app,
 			   GtkWidget *parent_widget,
 			   gint pos,
 			   GnomeUIInfo *menuinfo,
-			   GnomeUIBuilderData uidata)
+			   GnomeUIBuilderData *uidata)
 {
   int i;
   int has_stock_pixmaps = FALSE;
   int justify_right = FALSE;
+  GnomeUIBuilderData *orig_uidata;
+
+  /* store a pointer to the original uidata structure to use it for the
+     subtrees */
+  orig_uidata = uidata;
 
   /* first check if any of them use the stock pixmaps, because if
      they do we need to use gnome_stock_menu_item for all of them
@@ -80,17 +89,24 @@ gnome_app_do_menu_creation(GnomeApp *app,
       switch(menuinfo[i].type)
 	{
 
+  case GNOME_APP_UI_BUILDER_DATA:
+    /* set the builder data for the following entries. this builder data is used with all the
+       following items in this GnomeUIInfo array. it does not apply to the subtrees nor other
+       GnomeUIInfo arrays on the same level */
+    uidata = (GnomeUIBuilderData *)menuinfo[i].moreinfo;
+    break;
+
 	case GNOME_APP_UI_JUSTIFY_RIGHT:
 	  justify_right = TRUE;
 	  break;
 
 	case GNOME_APP_UI_HELP:
-	  gnome_app_add_help_menu_entries(app, parent_widget, pos, &menuinfo[i]);
+	  pos = gnome_app_add_help_menu_entries(app, parent_widget, pos, &menuinfo[i]);
 	  break;
 
 	case GNOME_APP_UI_RADIOITEMS:
-	  gnome_app_add_radio_menu_entries(app, parent_widget, pos,
-					   menuinfo[i].moreinfo, uidata);
+	  pos = gnome_app_add_radio_menu_entries(app, parent_widget, pos,
+                                           menuinfo[i].moreinfo, uidata);
 	  break;
 	  
 	case GNOME_APP_UI_SEPARATOR:
@@ -107,16 +123,54 @@ gnome_app_do_menu_creation(GnomeApp *app,
 		gtk_check_menu_item_set_show_toggle(GTK_CHECK_MENU_ITEM(menuinfo[i].widget), TRUE);
 		gtk_check_menu_item_set_state(GTK_CHECK_MENU_ITEM(menuinfo[i].widget), FALSE);
 	      }
-	    else
-	      if (has_stock_pixmaps)
-		menuinfo[i].widget =
-		  gnome_stock_menu_item((menuinfo[i].pixmap_info 
-					 ? menuinfo[i].pixmap_info 
-					 : GNOME_STOCK_MENU_BLANK),
-					_(menuinfo[i].label));
-	      else
-		menuinfo[i].widget = gtk_menu_item_new_with_label(_(menuinfo[i].label));
+	    else {
+		    GtkWidget *pmap = NULL;
+		    
+		    switch(menuinfo[i].pixmap_type) {
+		     case GNOME_APP_PIXMAP_DATA:
+			    pmap = gnome_pixmap_new_from_xpm_d(menuinfo[i].pixmap_info);
+			    break;
+		     case GNOME_APP_PIXMAP_FILENAME:
+			    pmap = gnome_pixmap_new_from_file(menuinfo[i].pixmap_info);
+			    break;
+		     case GNOME_APP_PIXMAP_NONE:
+		     case GNOME_APP_PIXMAP_STOCK:
+			    if (has_stock_pixmaps)
+			      menuinfo[i].widget =
+				gnome_stock_menu_item (menuinfo[i].pixmap_info ?
+						       menuinfo[i].pixmap_info :
+						       GNOME_STOCK_MENU_BLANK,
+						       _(menuinfo[i].label));
+			    else
+			      menuinfo[i].widget = gtk_menu_item_new_with_label(_(menuinfo[i].label));
+			    break;
+		     default:
+			    g_warning("GNOME_APP_PIXMAP type unknown.");
+			    break;
+		    }
+		    
+		    if (pmap) {
+			    GtkWidget *hbox, *w;
+			    
+			    hbox = gtk_hbox_new(FALSE, 2);
+			    gtk_box_pack_start(GTK_BOX(hbox), pmap, 
+					       FALSE, FALSE, 0);
+			    menuinfo[i].widget = gtk_menu_item_new();
+#ifdef	GTK_HAVE_FEATURES_1_1_0
+			    w = gtk_accel_label_new (_(menuinfo[i].label));
+			    gtk_accel_label_set_accel_widget (GTK_ACCEL_LABEL (w),
+							      menuinfo[i].widget);
+#else	/* !GTK_HAVE_FEATURES_1_1_0 */
+			    w = gtk_label_new(_(menuinfo[i].label));
+#endif	/* !GTK_HAVE_FEATURES_1_1_0 */
+			    gtk_misc_set_alignment (GTK_MISC (w), 0.0, 0.5);
+			    gtk_box_pack_start(GTK_BOX(hbox), w, TRUE, TRUE, 0);
+			    gtk_container_add(GTK_CONTAINER(menuinfo[i].widget), hbox);
+			    gtk_widget_show_all(hbox);
 
+		    }
+	    }
+		  
 	    gtk_widget_show(menuinfo[i].widget);
 
 	    if(justify_right)
@@ -140,7 +194,7 @@ gnome_app_do_menu_creation(GnomeApp *app,
 					   submenu,
 					   0,
 					   menuinfo[i].moreinfo,
-					   uidata);
+					   orig_uidata);
 	      }
 	  }
 	  break;
@@ -156,36 +210,48 @@ gnome_app_do_menu_creation(GnomeApp *app,
   menuinfo[i].widget = parent_widget;
 }
 
-static void
+static gint
 gnome_app_add_radio_menu_entries(GnomeApp *app,
 				 GtkWidget *parent_widget,
 				 gint pos,
 				 GnomeUIInfo *menuinfo,
-				 GnomeUIBuilderData uidata)
+				 GnomeUIBuilderData *uidata)
 {
   GSList *group = NULL;
 
-  g_return_if_fail(menuinfo != NULL);
+  g_return_val_if_fail(menuinfo != NULL, -1);
   while (menuinfo->type != GNOME_APP_UI_ENDOFINFO)
     {
-      menuinfo->widget = gtk_radio_menu_item_new_with_label(group, _(menuinfo->label));
-      group = gtk_radio_menu_item_group(GTK_RADIO_MENU_ITEM(menuinfo->widget));
-
-      gtk_check_menu_item_set_show_toggle(GTK_CHECK_MENU_ITEM(menuinfo->widget), TRUE);
-      gtk_check_menu_item_set_state(GTK_CHECK_MENU_ITEM(menuinfo->widget), FALSE);
-
-      gtk_widget_show(menuinfo->widget);
-      gtk_menu_shell_insert(GTK_MENU_SHELL(parent_widget), menuinfo->widget, pos);
-      pos++;
-
-      uidata->connect_func(app, menuinfo, "activate", uidata);
-      gnome_app_do_ui_accelerator_setup(app, "activate", menuinfo);
+      if (menuinfo->type == GNOME_APP_UI_SEPARATOR) {
+        menuinfo->widget = gtk_menu_item_new();
+        gtk_widget_show(menuinfo->widget);
+        gtk_menu_shell_insert(GTK_MENU_SHELL(parent_widget), menuinfo->widget, pos);
+        pos++;
+        menuinfo++;
+      }
+      else if (menuinfo->type == GNOME_APP_UI_BUILDER_DATA)
+        uidata = (GnomeUIBuilderData *)menuinfo->moreinfo;
+      else {
+        menuinfo->widget = gtk_radio_menu_item_new_with_label(group, _(menuinfo->label));
+        group = gtk_radio_menu_item_group(GTK_RADIO_MENU_ITEM(menuinfo->widget));
 	
-      menuinfo++;
+        gtk_check_menu_item_set_show_toggle(GTK_CHECK_MENU_ITEM(menuinfo->widget), TRUE);
+        gtk_check_menu_item_set_state(GTK_CHECK_MENU_ITEM(menuinfo->widget), FALSE);
+        gtk_widget_show(menuinfo->widget);
+        gtk_menu_shell_insert(GTK_MENU_SHELL(parent_widget), menuinfo->widget, pos);
+        pos++;
+
+        uidata->connect_func(app, menuinfo, "activate", uidata);
+        gnome_app_do_ui_accelerator_setup(app, "activate", menuinfo);
+
+        menuinfo++;
+      }
     }
+
+  return pos;
 }
 
-static void
+static gint
 gnome_app_add_help_menu_entries(GnomeApp *app,
 				GtkWidget *parent_widget,
 				gint pos,
@@ -207,7 +273,7 @@ gnome_app_add_help_menu_entries(GnomeApp *app,
       /* some standard help page                             */
 /*      fprintf(stderr, "Unable to open %s\n", (topicFile) ? topicFile : "(null)"); */
       g_free(topicFile);
-      return;
+      return -1;
     }
   g_free(topicFile);
 
@@ -234,30 +300,36 @@ gnome_app_add_help_menu_entries(GnomeApp *app,
       menuinfo_item->widget = gtk_menu_item_new_with_label(s);
       gtk_widget_show(menuinfo_item->widget);
       if(menuinfo_item->accelerator_key)
-	{
-	  gnome_app_do_ui_accelerator_setup(app, "activate", menuinfo_item);
-	  menuinfo_item->accelerator_key = 0;
-	}
+        {
+          gnome_app_do_ui_accelerator_setup(app, "activate", menuinfo_item);
+          menuinfo_item->accelerator_key = 0;
+        }
       gtk_menu_shell_insert(GTK_MENU_SHELL(parent_widget), menuinfo_item->widget, pos);
       pos++;
 
       gtk_signal_connect(GTK_OBJECT (menuinfo_item->widget), "activate",
 			 (gpointer) gnome_help_display,
-			 (gpointer) entry);
-    }
+                         (gpointer) entry);
+      }
   menuinfo_item->widget = NULL; /* We have a bunch of widgets,
 				   so this is useless ;-) */
     
   fclose(f);
+
+  return pos;
 }
 
 void
 gnome_app_create_menus_custom (GnomeApp *app,
 			       GnomeUIInfo *menuinfo,
-			       GnomeUIBuilderData uibdata)
+			       GnomeUIBuilderData *uibdata)
 {
   GtkWidget *menubar;
+#ifdef GTK_HAVE_FEATURES_1_1_0
+  GtkAccelGroup *ag;
+#else
   GtkAcceleratorTable *at;
+#endif
   int set_accel;
 
   g_return_if_fail(app != NULL);
@@ -266,15 +338,25 @@ gnome_app_create_menus_custom (GnomeApp *app,
 
   menubar = gtk_menu_bar_new ();
   gnome_app_set_menus (app, GTK_MENU_BAR (menubar));
-	
+#ifdef GTK_HAVE_FEATURES_1_1_0	
+  ag = gtk_object_get_data(GTK_OBJECT(app), "GtkAccelGroup");
+  set_accel = (ag == NULL);
+#else
   at = gtk_object_get_data(GTK_OBJECT(app), "GtkAcceleratorTable");
   set_accel = (at == NULL);
+#endif
   if (menuinfo)
     gnome_app_do_menu_creation(app, app->menubar, 0, menuinfo, uibdata);
   if (set_accel) {
+#ifdef GTK_HAVE_FEATURES_1_1_0
+    ag = gtk_object_get_data(GTK_OBJECT(app), "GtkAccelGroup");
+    if (ag && !g_slist_find(gtk_accel_groups_from_object (GTK_OBJECT (app)), ag))
+      gtk_window_add_accel_group(GTK_WINDOW(app), ag);
+#else
     at = gtk_object_get_data(GTK_OBJECT(app), "GtkAcceleratorTable");
     if (at)
       gtk_window_add_accelerator_table(GTK_WINDOW(app), at);
+#endif
   }
 }
 
@@ -282,8 +364,8 @@ void
 gnome_app_create_menus(GnomeApp *app,
 		       GnomeUIInfo *menuinfo)
 {
-  struct _GnomeUIBuilderData uidata = { GNOME_UISIGFUNC(gnome_app_do_ui_signal_connect),
-					NULL, FALSE, NULL, NULL};
+  GnomeUIBuilderData uidata = { GNOME_UISIGFUNC(gnome_app_do_ui_signal_connect),
+                                NULL, FALSE, NULL, NULL};
 
   gnome_app_create_menus_custom(app, menuinfo, &uidata);
 }
@@ -293,8 +375,8 @@ gnome_app_create_menus_with_data(GnomeApp *app,
 				 GnomeUIInfo *menuinfo,
 				 gpointer data)
 {
-  struct _GnomeUIBuilderData uidata = { GNOME_UISIGFUNC(gnome_app_do_ui_signal_connect),
-					NULL, FALSE, NULL, NULL };
+  GnomeUIBuilderData uidata = { GNOME_UISIGFUNC(gnome_app_do_ui_signal_connect),
+                                NULL, FALSE, NULL, NULL };
 	
   uidata.data = data;
 
@@ -308,8 +390,8 @@ gnome_app_create_menus_interp (GnomeApp *app,
 			       gpointer data,
 			       GtkDestroyNotify destroy_func)
 {
-  struct _GnomeUIBuilderData uidata = { GNOME_UISIGFUNC(gnome_app_do_ui_signal_connect),
-					NULL, FALSE, NULL, NULL };
+  GnomeUIBuilderData uidata = { GNOME_UISIGFUNC(gnome_app_do_ui_signal_connect),
+                                NULL, FALSE, NULL, NULL };
 
   uidata.data = data;
   uidata.is_interp = TRUE;
@@ -323,14 +405,22 @@ static void
 gnome_app_do_toolbar_creation(GnomeApp *app,
 			      GtkWidget *parent_widget,
 			      GnomeUIInfo *tbinfo,
-			      GnomeUIBuilderData uidata)
+			      GnomeUIBuilderData *uidata)
 {
   int i, set_accel;
-  GtkAcceleratorTable *at;
   GtkWidget *pmap;
+#ifdef GTK_HAVE_FEATURES_1_1_0
+  GtkAccelGroup *ag;
 
   set_accel = (NULL == gtk_object_get_data(GTK_OBJECT(app),
-					   "GtkAcceleratorTable"));
+					   "GtkAccelGroup"));
+#else
+  GtkAcceleratorTable *at;
+
+  set_accel = (NULL == gtk_object_get_data(GTK_OBJECT(app),
+                                           "GtkAcceleratorTable"));
+#endif
+
   if(!GTK_WIDGET(app)->window)
     gtk_widget_realize(GTK_WIDGET(app));
 	
@@ -338,6 +428,13 @@ gnome_app_do_toolbar_creation(GnomeApp *app,
     {
       switch(tbinfo[i].type)
 	{
+  case GNOME_APP_UI_BUILDER_DATA:
+    /* set the builder data for the following entries. this builder data is used with all the
+       following items in this GnomeUIInfo array. it does not apply to the subtrees nor other
+       GnomeUIInfo arrays on the same level */
+    uidata = (GnomeUIBuilderData *)tbinfo[i].moreinfo;
+    break;
+
 	case GNOME_APP_UI_RADIOITEMS:
 	  gnome_app_add_radio_toolbar_entries(app, parent_widget,
 					      tbinfo[i].moreinfo, uidata);
@@ -390,9 +487,15 @@ gnome_app_do_toolbar_creation(GnomeApp *app,
     }
   tbinfo[i].widget = parent_widget;
   if (set_accel) {
+#ifdef GTK_HAVE_FEATURES_1_1_0
+    ag = gtk_object_get_data(GTK_OBJECT(app), "GtkAccelGroup");
+    if (ag && !g_slist_find(gtk_accel_groups_from_object (GTK_OBJECT (app)), ag))
+      gtk_window_add_accel_group(GTK_WINDOW(app), ag);
+#else
     at = gtk_object_get_data(GTK_OBJECT(app), "GtkAcceleratorTable");
     if (at)
       gtk_window_add_accelerator_table(GTK_WINDOW(app), at);
+#endif
   }
 }
 
@@ -400,7 +503,7 @@ static void
 gnome_app_add_radio_toolbar_entries(GnomeApp *app,
 				    GtkWidget *parent_widget,
 				    GnomeUIInfo *tbinfo,
-				    GnomeUIBuilderData uidata)
+				    GnomeUIBuilderData *uidata)
 {
   GtkWidget *group = NULL;
   GtkWidget *pmap;
@@ -409,42 +512,46 @@ gnome_app_add_radio_toolbar_entries(GnomeApp *app,
     
   while (tbinfo->type != GNOME_APP_UI_ENDOFINFO)
     {
-      switch(tbinfo->pixmap_type) {
-      case GNOME_APP_PIXMAP_DATA:
-	pmap = gnome_pixmap_new_from_xpm_d(tbinfo->pixmap_info);
-	break;
+      if(tbinfo->type == GNOME_APP_UI_BUILDER_DATA)
+         uidata = (GnomeUIBuilderData *)tbinfo->moreinfo;
+      else {
+        switch(tbinfo->pixmap_type) {
+        case GNOME_APP_PIXMAP_DATA:
+          pmap = gnome_pixmap_new_from_xpm_d(tbinfo->pixmap_info);
+          break;
 
-      case GNOME_APP_PIXMAP_FILENAME:
-	pmap = gnome_pixmap_new_from_file((char *)tbinfo->pixmap_info);
-	break;
+        case GNOME_APP_PIXMAP_FILENAME:
+          pmap = gnome_pixmap_new_from_file((char *)tbinfo->pixmap_info);
+          break;
 
-      case GNOME_APP_PIXMAP_STOCK:
-	pmap = gnome_stock_pixmap_widget_new(GTK_WIDGET(app), tbinfo->pixmap_info);
-	break;
+        case GNOME_APP_PIXMAP_STOCK:
+          pmap = gnome_stock_pixmap_widget_new(GTK_WIDGET(app), tbinfo->pixmap_info);
+          break;
 
-      default:
-	pmap = NULL; break;
-      }
+        default:
+          pmap = NULL; break;
+        }
 
-      group = tbinfo->widget =
-	gtk_toolbar_append_element(GTK_TOOLBAR(parent_widget),
-				   GTK_TOOLBAR_CHILD_RADIOBUTTON,
-				   group, _(tbinfo->label),
-				   _(tbinfo->hint), NULL,
-				   pmap, NULL, NULL);
+        group = tbinfo->widget =
+          gtk_toolbar_append_element(GTK_TOOLBAR(parent_widget),
+                                     GTK_TOOLBAR_CHILD_RADIOBUTTON,
+                                     group, _(tbinfo->label),
+                                     _(tbinfo->hint), NULL,
+                                     pmap, NULL, NULL);
 
-      uidata->connect_func(app, tbinfo, "clicked", uidata);
+        uidata->connect_func(app, tbinfo, "clicked", uidata);
 
-      gnome_app_do_ui_accelerator_setup(app, "clicked", tbinfo);
+        gnome_app_do_ui_accelerator_setup(app, "clicked", tbinfo);
 	
-      tbinfo++;
+        tbinfo++;
+      }
     }
 }
 
 void
 gnome_app_create_toolbar_custom (GnomeApp *app,
 				 GnomeUIInfo *tbinfo,
-				 GnomeUIBuilderData uibdata)
+				 GnomeUIBuilderData *uibdata)
 {
   GtkWidget *tb;
 
@@ -463,8 +570,8 @@ void
 gnome_app_create_toolbar(GnomeApp *app,
 			 GnomeUIInfo *toolbarinfo)
 {
-  struct _GnomeUIBuilderData uidata = { GNOME_UISIGFUNC(gnome_app_do_ui_signal_connect),
-					NULL, FALSE, NULL, NULL };
+  GnomeUIBuilderData uidata = { GNOME_UISIGFUNC(gnome_app_do_ui_signal_connect),
+                                NULL, FALSE, NULL, NULL };
 
   gnome_app_create_toolbar_custom(app, toolbarinfo, &uidata);
 }
@@ -474,8 +581,8 @@ gnome_app_create_toolbar_with_data(GnomeApp *app,
 				   GnomeUIInfo *toolbarinfo,
 				   gpointer data)
 {
-  struct _GnomeUIBuilderData uidata = { GNOME_UISIGFUNC(gnome_app_do_ui_signal_connect),
-					NULL, FALSE, NULL, NULL };
+  GnomeUIBuilderData uidata = { GNOME_UISIGFUNC(gnome_app_do_ui_signal_connect),
+                                NULL, FALSE, NULL, NULL };
 
   uidata.data = data;
   gnome_app_create_toolbar_custom(app, toolbarinfo, &uidata);
@@ -488,8 +595,8 @@ gnome_app_create_toolbar_interp (GnomeApp *app,
 				 gpointer data,
 				 GtkDestroyNotify destroy_func)
 {
-  struct _GnomeUIBuilderData uidata = { GNOME_UISIGFUNC(gnome_app_do_ui_signal_connect),
-					NULL, FALSE, NULL, NULL };
+  GnomeUIBuilderData uidata = { GNOME_UISIGFUNC(gnome_app_do_ui_signal_connect),
+                                NULL, FALSE, NULL, NULL };
 
   uidata.data = data;
   uidata.is_interp = TRUE;
@@ -499,11 +606,11 @@ gnome_app_create_toolbar_interp (GnomeApp *app,
   gnome_app_create_toolbar_custom(app, tbinfo, &uidata);
 }
 
-static void
+void
 gnome_app_do_ui_signal_connect (GnomeApp *app,
 				GnomeUIInfo *info_item,
 				gchar *signal_name,
-				GnomeUIBuilderData uidata)
+				GnomeUIBuilderData *uidata)
 {
   if(uidata->is_interp)
     gtk_signal_connect_interp(GTK_OBJECT(info_item->widget),
@@ -523,25 +630,46 @@ gnome_app_do_ui_accelerator_setup (GnomeApp *app,
 				   gchar *signal_name,
 				   GnomeUIInfo *menuinfo_item)
 {
+#ifdef GTK_HAVE_FEATURES_1_1_0
+  GtkAccelGroup *ag;
+#else
   GtkAcceleratorTable *at;
-
+#endif
   if (menuinfo_item->accelerator_key == 0)
     return;
+#ifdef GTK_HAVE_FEATURES_1_1_0
+  ag = gtk_object_get_data(GTK_OBJECT(app),
+			   "GtkAccelGroup");
+  if (ag == NULL)
+    {
+      ag = gtk_accel_group_new();
+      gtk_object_set_data(GTK_OBJECT(app),
+			  "GtkAccelGroup",
+			  (gpointer)ag);
+    }
 
+  gtk_widget_add_accelerator(GTK_WIDGET(menuinfo_item->widget),
+				   signal_name,
+				   ag,
+				   menuinfo_item->accelerator_key,
+				   menuinfo_item->ac_mods,
+				   GTK_ACCEL_VISIBLE);
+#else
   at = gtk_object_get_data(GTK_OBJECT(app),
-			   "GtkAcceleratorTable");
+                           "GtkAcceleratorTable");
   if (at == NULL)
     {
       at = gtk_accelerator_table_new();
       gtk_object_set_data(GTK_OBJECT(app),
-			  "GtkAcceleratorTable",
-			  (gpointer)at);
+                          "GtkAcceleratorTable",
+                          (gpointer)at);
     }
 
   gtk_widget_install_accelerator(GTK_WIDGET(menuinfo_item->widget), at,
-				   signal_name,
-				   menuinfo_item->accelerator_key,
-				   menuinfo_item->ac_mods);
+                                   signal_name,
+                                   menuinfo_item->accelerator_key,
+                                   menuinfo_item->ac_mods);
+#endif
 }
 
 /*
@@ -552,6 +680,7 @@ gnome_app_do_ui_accelerator_setup (GnomeApp *app,
  * "" will insert the item as the first one in the menubar
  * "File/" will insert it as the first one in the File menu
  * "File/Settings" will insert it after the Setting item in the File menu
+ * use of  "File/<Separator>" should be obvious. however this stops after the first separator.
  * I hope this explains use of the insert/remove functions well enough.
  */
 
@@ -568,7 +697,7 @@ gnome_app_find_menu_pos (GtkWidget *parent,
 			 gint *pos)
 {
   GtkBin *item;
-  GtkLabel *label = NULL;
+  gchar *label = NULL;
   GList *children;
   gchar *name_end;
   gint p, path_len;
@@ -596,22 +725,24 @@ gnome_app_find_menu_pos (GtkWidget *parent,
     p++;
     item = GTK_BIN(children->data);
 
-    if(!item->child)
-      label = NULL;
-    else if(GTK_IS_LABEL(item->child))
-      label = GTK_LABEL(item->child);
-    else if(GTK_IS_HBOX(item->child))
-      label = GTK_LABEL(g_list_next(gtk_container_children(GTK_CONTAINER(item->child)))->data);
-    else
+    if(!item->child)                    /* this is a separator, right? */
+      label = "<Separator>";
+    else if(GTK_IS_LABEL(item->child))  /* a simple item with a label */
+      label = GTK_LABEL(item->child)->label;
+    else if(GTK_IS_HBOX(item->child))   /* an item with a hbox (pixmap + label) */
+      label = GTK_LABEL(g_list_next(gtk_container_children(GTK_CONTAINER(item->child)))->data)->label;
+    else                                /* something that we just can't handle */
       label = NULL;
 
-    if( label && (strncmp(path, label->label, path_len) == 0) ) {
+    if( label && (path_len == strlen(label)) && (strncmp(path, label, path_len) == 0) ) {
       if(name_end == NULL) {
-	*pos = p;
-	return parent;
+        *pos = p;
+        return parent;
       }
+      else if(GTK_MENU_ITEM(item)->submenu)
+        return gnome_app_find_menu_pos(GTK_MENU_ITEM(item)->submenu, (gchar *)(name_end + 1), pos);
       else
-	return gnome_app_find_menu_pos(GTK_MENU_ITEM(item)->submenu, (gchar *)(name_end + 1), pos);
+        return NULL;
     }
 
     children = g_list_next(children);
@@ -669,10 +800,12 @@ void
 gnome_app_insert_menus_custom (GnomeApp *app,
 			       gchar *path,
 			       GnomeUIInfo *menuinfo,
-			       GnomeUIBuilderData uibdata)
+			       GnomeUIBuilderData *uibdata)
 {
   GtkWidget *parent;
+#ifndef GTK_HAVE_FEATURES_1_1_0
   GtkAcceleratorTable *at;
+#endif
   gint pos;
 
   g_return_if_fail(app != NULL);
@@ -702,8 +835,8 @@ gnome_app_insert_menus(GnomeApp *app,
 		       gchar *path,
 		       GnomeUIInfo *menuinfo)
 {
-  struct _GnomeUIBuilderData uidata = { GNOME_UISIGFUNC(gnome_app_do_ui_signal_connect),
-					NULL, FALSE, NULL, NULL};
+  GnomeUIBuilderData uidata = { GNOME_UISIGFUNC(gnome_app_do_ui_signal_connect),
+                                NULL, FALSE, NULL, NULL};
 
   gnome_app_insert_menus_custom(app, path, menuinfo, &uidata);
 }
@@ -714,8 +847,8 @@ gnome_app_insert_menus_with_data(GnomeApp *app,
 				 GnomeUIInfo *menuinfo,
 				 gpointer data)
 {
-  struct _GnomeUIBuilderData uidata = { GNOME_UISIGFUNC(gnome_app_do_ui_signal_connect),
-					NULL, FALSE, NULL, NULL };
+  GnomeUIBuilderData uidata = { GNOME_UISIGFUNC(gnome_app_do_ui_signal_connect),
+                                NULL, FALSE, NULL, NULL };
 	
   uidata.data = data;
 
@@ -730,8 +863,8 @@ gnome_app_insert_menus_interp (GnomeApp *app,
 			       gpointer data,
 			       GtkDestroyNotify destroy_func)
 {
-  struct _GnomeUIBuilderData uidata = { GNOME_UISIGFUNC(gnome_app_do_ui_signal_connect),
-					NULL, FALSE, NULL, NULL };
+  GnomeUIBuilderData uidata = { GNOME_UISIGFUNC(gnome_app_do_ui_signal_connect),
+                                NULL, FALSE, NULL, NULL };
 
   uidata.data = data;
   uidata.is_interp = TRUE;
