@@ -3,13 +3,14 @@
  * Author: George Lebl <jirka@5z.com>
  */
 
+#include <config.h>
 #include <stdio.h>
 #include <string.h>
 #include <math.h>
 #include <errno.h>
 #include <signal.h>
 #include <gtk/gtk.h>
-#include "libgnome/libgnome.h"
+#include "libgnome/libgnomeP.h"
 #include "gnome-calculator.h"
 
 typedef void (*sighandler_t)(int);
@@ -165,6 +166,9 @@ put_led_font(GnomeCalculator *gc)
 	gint i;
 	gint x;
 
+	if(!GTK_WIDGET_REALIZED(GTK_WIDGET(gc)))
+		return;
+
 	style = gtk_widget_get_style(gc->display);
 	gtk_pixmap_get(GTK_PIXMAP(gc->display), &p, NULL);
 
@@ -570,6 +574,15 @@ clear_calc(GtkWidget *w, gpointer data)
 	return TRUE;
 }
 
+void
+gnome_calculator_clear(GnomeCalculator *gc, gint reset)
+{
+	if(reset)
+		reset_calc(NULL,gc);
+	else
+		clear_calc(NULL,gc);
+}
+
 static gint
 add_digit(GtkWidget *w, gpointer data)
 {
@@ -769,6 +782,12 @@ set_result_to(GnomeCalculator *gc, gdouble result)
 	return old;
 }
 
+void
+gnome_calculator_set(GnomeCalculator *gc, gdouble result)
+{
+	set_result_to(gc,result);
+}
+
 static gint
 store_m(GtkWidget *w, gpointer data)
 {
@@ -1101,7 +1120,11 @@ gnome_calculator_init (GnomeCalculator *gc)
 	gc->mode = GNOME_CALCULATOR_DEG;
 	gc->invert = FALSE;
 	gc->add_digit = FALSE;
+#ifdef GTK_HAVE_FEATURES_1_1_0
+	gc->accel = gtk_accel_group_new();
+#else
 	gc->accel = gtk_accelerator_table_new();
+#endif
 
 	gtk_signal_connect_after(GTK_OBJECT(gc),"realize",
 				 GTK_SIGNAL_FUNC(gnome_calculator_realized),
@@ -1120,22 +1143,44 @@ gnome_calculator_init (GnomeCalculator *gc)
 				gtk_signal_connect(GTK_OBJECT(w),"clicked",
 						   but->signal_func,
 						   but);
+#ifdef GTK_HAVE_FEATURES_1_1_0
 				if(but->key) {
-					gtk_widget_install_accelerator(w,
-						gc->accel,
-						"clicked",
-						but->key,0);
-					gtk_widget_install_accelerator(w,
-						gc->accel,
-						"clicked",
-						but->key,
-						GDK_SHIFT_MASK);
-					gtk_widget_install_accelerator(w,
-						gc->accel,
-						"clicked",
-						but->key,
-						GDK_LOCK_MASK);
+				  gtk_widget_add_accelerator(w,
+							     "clicked",
+							     gc->accel,
+							     but->key,0,
+							     GTK_ACCEL_VISIBLE);
+				  gtk_widget_add_accelerator(w,
+							     "clicked",
+							     gc->accel,
+							     but->key,
+							     GDK_SHIFT_MASK,
+							     GTK_ACCEL_VISIBLE);
+				  gtk_widget_add_accelerator(w,
+							     "clicked",
+							     gc->accel,
+							     but->key,
+							     GDK_LOCK_MASK,
+							     GTK_ACCEL_VISIBLE);
 				}
+#else
+                                if(but->key) {
+                                        gtk_widget_install_accelerator(w,
+                                                gc->accel,
+                                                "clicked",
+                                                but->key,0);
+                                        gtk_widget_install_accelerator(w,
+                                                gc->accel,
+                                                "clicked",
+                                                but->key,
+                                                GDK_SHIFT_MASK);
+                                        gtk_widget_install_accelerator(w,
+                                                gc->accel,
+                                                "clicked",
+                                                but->key,
+                                                GDK_LOCK_MASK);
+				}
+#endif
 				gtk_object_set_user_data(GTK_OBJECT(w),gc);
 				gtk_widget_show(w);
 				gtk_table_attach(GTK_TABLE(table),w,
