@@ -6,9 +6,9 @@
 #ifndef GNOME_CALCULATOR_H
 #define GNOME_CALCULATOR_H
 
-
 #include <gdk/gdk.h>
 #include <gtk/gtkvbox.h>
+#include <gtk/gtkfeatures.h>
 #include <libgnome/gnome-defs.h>
 
 BEGIN_GNOME_DECLS
@@ -46,8 +46,11 @@ struct _GnomeCalculator {
 	GtkWidget *invert_button;
 
 	GList *stack;
-
+#ifdef GTK_HAVE_FEATURES_1_1_0
+	GtkAccelGroup *accel;
+#else
 	GtkAcceleratorTable *accel;
+#endif
 };
 
 struct _GnomeCalculatorClass {
@@ -60,7 +63,12 @@ struct _GnomeCalculatorClass {
 
 guint		 gnome_calculator_get_type	(void);
 GtkWidget	*gnome_calculator_new		(void);
-void		 gnome_calculator_clear		(void);
+void		 gnome_calculator_clear		(GnomeCalculator *gc,
+						 gint reset);
+void		 gnome_calculator_set		(GnomeCalculator *gc,
+						 gdouble result);
+
+#define gnome_calculator_get_result(gc) (GNOME_CALCULATOR(gc)->result)
 
 END_GNOME_DECLS
 
