@@ -6,6 +6,7 @@
 #include <gtk/gtkwidget.h>
 #include <gdk_imlib.h>
 
+#include <libgnome/gnome-defs.h>
 
 BEGIN_GNOME_DECLS
 
@@ -32,8 +33,8 @@ struct _GnomePixmapClass {
 
 guint      gnome_pixmap_get_type               (void);
 
-GtkWidget *gnome_pixmap_new_from_file          (char *filename);
-GtkWidget *gnome_pixmap_new_from_file_at_size  (char *filename, int width, int height);
+GtkWidget *gnome_pixmap_new_from_file          (const char *filename);
+GtkWidget *gnome_pixmap_new_from_file_at_size  (const char *filename, int width, int height);
 GtkWidget *gnome_pixmap_new_from_xpm_d         (char **xpm_data);
 GtkWidget *gnome_pixmap_new_from_xpm_d_at_size (char **xpm_data, int width, int height);
 GtkWidget *gnome_pixmap_new_from_rgb_d         (unsigned char *data, unsigned char *alpha,
@@ -45,9 +46,12 @@ GtkWidget *gnome_pixmap_new_from_rgb_d_at_size (unsigned char *data, unsigned ch
 						int rgb_width, int rgb_height,
 						int width, int height);
 GtkWidget *gnome_pixmap_new_from_gnome_pixmap  (GnomePixmap *gpixmap);
+GtkWidget *gnome_pixmap_new_from_imlib         (GdkImlibImage *im);
+GtkWidget *gnome_pixmap_new_from_imlib_at_size (GdkImlibImage *im,
+						int width, int height);
 
-void       gnome_pixmap_load_file              (GnomePixmap *gpixmap, char *filename);
-void       gnome_pixmap_load_file_at_size      (GnomePixmap *gpixmap, char *filename, int width, int height);
+void       gnome_pixmap_load_file              (GnomePixmap *gpixmap, const char *filename);
+void       gnome_pixmap_load_file_at_size      (GnomePixmap *gpixmap, const char *filename, int width, int height);
 void       gnome_pixmap_load_xpm_d             (GnomePixmap *gpixmap, char **xpm_data);
 void       gnome_pixmap_load_xpm_d_at_size     (GnomePixmap *gpixmap, char **xpm_data, int width, int height);
 void       gnome_pixmap_load_rgb_d             (GnomePixmap *gpixmap, unsigned char *data, unsigned char *alpha,
@@ -58,44 +62,11 @@ void       gnome_pixmap_load_rgb_d_shaped      (GnomePixmap *gpixmap, unsigned c
 void       gnome_pixmap_load_rgb_d_at_size     (GnomePixmap *gpixmap, unsigned char *data, unsigned char *alpha,
 						int rgb_width, int rgb_height,
 						int width, int height);
-
-
-
-/* FIXME:  These are the old gnome-pixmap API functions.  They should
- * be flushed out in favor of the GnomePixmap widget.
- */
-
-void gnome_destroy_pixmap_cache (void);
-
-void gnome_create_pixmap_gdk              (GdkWindow *window,
-					   GdkPixmap **pixmap,
-					   GdkBitmap **mask,
-					   GdkColor  *transparent,
-					   char *file);
-void gnome_create_pixmap_gtk              (GtkWidget *window,
-					   GdkPixmap **pixmap,
-					   GdkBitmap **mask,
-					   GtkWidget *holder,
-					   char *file);
-void gnome_create_pixmap_gtk_d            (GtkWidget *window,
-					   GdkPixmap **pixmap,
-					   GdkBitmap **mask,
-					   GtkWidget *holder,
-					   char **data);
-GtkWidget *gnome_create_pixmap_widget     (GtkWidget *window,
-					   GtkWidget *holder,
-					   char *file);
-GtkWidget *gnome_create_pixmap_widget_d   (GtkWidget *window,
-					   GtkWidget *holder,
-					   char **data);
-void gnome_set_pixmap_widget              (GtkPixmap *pixmap,	
-					   GtkWidget *window,
-					   GtkWidget *holder,
-					   char *file);
-void gnome_set_pixmap_widget_d            (GtkPixmap *pixmap,	
-					   GtkWidget *window,
-					   GtkWidget *holder,
-					   char **data);
+void       gnome_pixmap_load_imlib             (GnomePixmap *gpixmap,
+						GdkImlibImage *im);
+void       gnome_pixmap_load_imlib_at_size     (GnomePixmap *gpixmap,
+				                GdkImlibImage *im,
+						int width, int height);
 
 
 END_GNOME_DECLS
